{
    "$graph": [
        {
            "class": "CommandLineTool",
            "label": "Compute and write average structure for given set of structures",
            "doc": "The tool chops the PDB files into domains according to given residue numebring and average structure from all chopped structures.\nThe input file is a json file where key is family id with and value is list of domain structures with residue numbering.\n",
            "requirements": [
                {
                    "envDef": [
                        {
                            "envValue": "$(inputs.kpax_result)",
                            "envName": "KPAX_RESULTS"
                        }
                    ],
                    "class": "EnvVarRequirement"
                },
                {
                    "listing": [
                        {
                            "entryname": "align_compute_avg.py",
                            "entry": "#!/usr/bin/env python3\n\"\"\"\nThis script aligns all the chooped structures using Kpax. Then the aligned structures are used\nto compute the average structure. This script requires a json file where key is family id with\nand value is list of domain structures with residue numbering.\n\"\"\"\n\nimport argparse\nimport warnings\nimport os\nimport sys\nimport Bio.PDB as bpdb\nimport pandas as pd\nwarnings.filterwarnings(\"ignore\")\n\nclass AvgStructComputation:\n    \"\"\"Class to compute average structure from given n number of 3-D structures,\n    without the need to have same number of atoms or residues\"\"\"\n\n    def __init__(self, fasta, a_dir, out):\n        self.fasta_file = fasta\n        self.struct_dir = a_dir\n        self.out_file = out\n        self.backbone = ['N', 'CA', 'C', 'O']\n\n\n    def compute_avg_struct(self):\n        \"\"\"\n        Computes average structure from a given set of structures\n        \"\"\"\n        dict3_1 = {'CYS': 'C', 'ASP': 'D', 'SER': 'S', 'GLN': 'Q', 'LYS': 'K', 'ILE': 'I', \\\n                   'PRO': 'P', 'THR': 'T', 'PHE': 'F', 'ASN': 'N', 'GLY': 'G', 'HIS': 'H', \\\n                   'LEU': 'L', 'ARG': 'R', 'TRP': 'W', 'ALA': 'A', 'VAL': 'V', 'GLU': 'E', \\\n                   'TYR': 'Y', 'MET': 'M', 'MSE': 'M'}\n        dict1_3 = {'C': 'CYS', 'D': 'ASP', 'S': 'SER', 'Q': 'GLN', 'K': 'LYS', 'I': 'ILE', \\\n                   'P': 'PRO', 'T': 'THR', 'F': 'PHE', 'N': 'ASN', 'G': 'GLY', 'H': 'HIS', \\\n                   'L': 'LEU', 'R': 'ARG', 'W': 'TRP', 'A': 'ALA', 'V': 'VAL', 'E': 'GLU', \\\n                   'Y': 'TYR', 'M': 'MET'}\n\n        seq = self.fasta_reader(self.fasta_file)  # Read fasta file\n\n        # setting of the position to be considered based on percentage of gaps;\n        # value ranges from 0 to 1\n        fraction = 0.5\n        for col in range(1, len(seq.loc[0])):\n            pdb_names = {}\n            list_x = list(seq.loc[0])\n            for pos in range(len(seq)):\n                # getting residue position in PDB\n                row_m = list(seq.loc[pos])[:col + 1]\n\n                if row_m[-1] == '-':\n                    continue\n                if '-' in row_m:\n                    row_m = [val for val in row_m if val != '-']\n\n                pdb_names[row_m[0]] = {len(row_m) - 1: row_m[-1]}\n            coor_res = self.get_coord_mean(pdb_names, self.struct_dir)\n\n            if self.most_frequent(list(seq[col])) == '-':\n                if list(seq[col]).count('-') > fraction * len(list(seq[col])):\n                    continue\n                else:\n                    res_x = list(seq[col])\n                    res_x = [val for val in res_x if val != '-']\n                    res_x = dict1_3[self.most_frequent(res_x)]\n            else:\n                res_x = dict1_3[self.most_frequent(list(seq[col]))]\n\n            with open(self.out_file, 'a') as wrtav:  # Writing the average structure\n                line = self.get_pdb_line(col, res_x, coor_res)\n                wrtav.write('\\n' + '\\n'.join(line))\n        return self.out_file\n\n    @staticmethod\n    def fasta_reader(filename):\n        \"\"\"\n        Reads the fasta file and returns all sequneces in a dataframe with first column as headers\n        \"\"\"\n        all_seq = open(filename).read().split('>')\n        all_seq = [nn.replace('\\n', '') for nn in all_seq[1:]]\n        all_seq = [nn.split() for nn in all_seq]\n        with open(filename.replace('.fasta', '.tmp'), 'w') as wrtfs:\n            for pos in all_seq:\n                wrtfs.write('\\n')\n                wrtfs.write(pos[0] + ',' + ','.join(list(pos[1])[1:]))\n\n        align_df = pd.read_csv(filename.replace('.fasta', '.tmp'), header=None)\n        return align_df\n\n    def get_coord_mean(self, pdb_dict, path):\n        \"\"\"\n        Computes average position for all backbone atoms (N, CA, C, O) & returns average positions\n        \"\"\"\n        bb_coor = {}\n        parsepdb = bpdb.PDBParser()\n\n        for pdb in pdb_dict:\n            # kpax retunrs .pdb extension with name only if only 2 structures are passed to align\n            pdbname = pdb if pdb.endswith('.pdb') else pdb + '.pdb'\n            # reading each structure\n            structure = parsepdb.get_structure(\"struct\", path + pdbname)\n            for chain in structure[0]:\n                num_n = 1\n                for residue in chain:\n                    if list(pdb_dict[pdb].keys())[0] == num_n:\n                        x_res = residue\n                        break\n                    num_n += 1\n\n                for atom in x_res:\n                    if atom.get_name() in self.backbone:\n                        if atom.get_name() in bb_coor.keys():\n                            bb_coor[atom.get_name()].append(list(atom.get_vector()))\n                        else:\n                            bb_coor[atom.get_name()] = [list(atom.get_vector())]\n\n        avg_coor = {}\n        for atom in bb_coor:\n            atom_coor = bb_coor[atom]\n            avg_coor[atom] = [sum(nn)/len(nn) for nn in zip(*atom_coor)]\n        return avg_coor\n\n    @staticmethod\n    def most_frequent(query_list):\n        \"\"\"Returns most frequent value from query list\"\"\"\n        return max(set(query_list), key=query_list.count)\n\n\n    def get_pdb_line(self, n, res, avg_coor):\n        \"\"\"Create the lines to be written for Average PDB structure \"\"\"\n        lines = []\n        for x in self.backbone:\n            if x not in avg_coor.keys():\n                continue\n            num = (n-1)*4\n            record = 'ATOM' + ' '*(7 - len(str(num + 1 + self.backbone.index(x))))\n            record += str(num + 1 + self.backbone.index(x)) + ' '*2 + x\n            record += ' '*(17 - len(record)) + res + ' ' + 'A'\n            record += ' '*(4 - len(str(n))) + str(n) + ' '*(12 - len(\"{:.3f}\".format(avg_coor[x][0])))\n            record += \"{:.3f}\".format(avg_coor[x][0]) + ' '*(8 - len(\"{:.3f}\".format(avg_coor[x][1])))\n            record += \"{:.3f}\".format(avg_coor[x][1]) + ' '*(8 - len(\"{:.3f}\".format(avg_coor[x][2])))\n            record += \"{:.3f}\".format(avg_coor[x][2])\n\n            lines.append(record)\n        return lines\n\n\nclass UtilityDir:\n    \"\"\"Class to provide some utilities for PDB files\"\"\"\n    def __init__(self, a_dir):\n        self.dir = a_dir\n\n    def recursively_remove_files(self):\n        \"\"\"USE WITH CAUTION:\n        Removes the directory even if it's not empty\"\"\"\n        if os.path.isfile(self.dir):\n            os.remove(self.dir)\n        elif os.path.isdir(self.dir):\n            for sub in os.listdir(self.dir):\n                self.recursively_remove_files(os.path.join(self.dir, sub))\n            os.rmdir(self.dir)\n\n    def empty_dir(self):\n        \"\"\"Remove all files from a directory\"\"\"\n        for afile in os.listdir(self.dir):\n            try:\n                os.remove(self.dir + afile)\n            except:\n                print('Unable to delete ', afile)\n\n\nclass Kpax:\n    \"\"\"Class for aligning structurs with Kpax; Currently only supports multiple alignment\"\"\"\n    def __init__(self, a_dir):\n        self.dir = a_dir\n\n    def kpax_multi(self):\n        \"\"\"\n        Multiple alignment using Kpax\n        \"\"\"\n        log = 'kpax_log1.tmp'\n        if (len(os.listdir(self.dir)) < 3) and (len(os.listdir(self.dir)) > 1):\n            # For only 2 PDB files\n            cmd = 'kpax5.1.3.x64 -broken -nowrite -pdb -fasta ' + \\\n                  os.path.join(self.dir, '*.pdb') + ' > ' + log\n        else:\n            # For more than two 2 PDB files\n            cmd = 'kpax5.1.3.x64 -multi -broken -nowrite -pdb -fasta ' + \\\n                  os.path.join(self.dir, '*.pdb') + ' > ' + log\n        os.system(cmd)\n        resdir, fasta = '', ''\n\n        for line in open(log, 'r'):\n            if line.startswith('Creating RESULTS directory'):\n                resdir = line.split(':')[1].replace(' ', '').replace('\\n', '')\n            elif line.startswith('Writing FASTA multiple alignment file'):\n                fasta = line.split(':')[1].replace(' ', '').replace('\\n', '')\n            if (len(os.listdir(self.dir)) < 3) and line.startswith('Writing TARGET pdb file'):\n                fasta = line.split(':')[1].replace(' ', '').replace('\\n', '')\n                fasta = fasta.replace('.pdb', '.fasta')\n\n            if (resdir != '') and (fasta != ''):\n                break\n        os.remove(log)\n        return resdir + '/', fasta\n\n    def __str__(self):\n        return \"Aligning all the structures from directory: {0}\".format(self.dir)\n\n\n###################################################################################################\nif __name__ == '__main__':\n    P = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter, \\\n        epilog='Enjoy the program! :)')\n    P.add_argument(\"-f\", \"--file\", type=str, required=True, \\\n        help=\"The file with instance name to be assigned to average structure\")\n    P.add_argument(\"-s\", \"--splitdir\", type=str, default=\"split_PDB/\", \\\n        help=\"The directory path for the PDB structures to be averaged\")\n    P.add_argument(\"-k\", \"--kpax_res\", type=str, default=\"KPAX_RESULTS/\", \\\n        help=\"The directory path where all the kpax alignment results will be stored\")\n\n    ARGS = P.parse_args()\n\n    a = open(ARGS.file).read().replace('\\n', '')\n\n    # Default directory for KPAX_RESULTS\n    if 'KPAX_RESULTS' not in os.environ:\n        os.environ['KPAX_RESULTS'] = ARGS.kpax_res\n    os.environ['KPAX_RESULTS'] = os.environ['KPAX_RESULTS'] if os.environ['KPAX_RESULTS'].endswith('/') else os.environ['KPAX_RESULTS'] + '/'\n\n    if os.path.exists(ARGS.kpax_res):\n        util_d = UtilityDir(ARGS.kpax_res)\n        util_d.recursively_remove_files()\n\n    if len(os.listdir(ARGS.splitdir)) > 1:\n        kp = Kpax(ARGS.splitdir)\n        kpax_dr, fastafile = kp.kpax_multi()\n    else:\n        if '.' in str(a):       # The change is bcz of kpax; Sometimes it replaces . with ~\n            a = str(a).replace('.', '-')            # To avoid the change in name\n        pdbout = str(a) + '_avgStruct.pdb'              # Name of the average PDB structure file\n\n        if len(os.listdir(ARGS.splitdir)) < 1:\n            print(\"\\n\\nNo PDB files in following Dir\", ARGS.splitdir, ARGS.file, a)\n            sys.exit()\n        os.system('mv '+ os.path.join(ARGS.splitdir, os.listdir(ARGS.splitdir)[0]) + ' '+ pdbout)\n        sys.exit()\n\n    if '.' in str(a):                # The change is bcz of kpax; Sometimes it replaces . with ~\n        a = str(a).replace('.', '-')            # To avoid the change in name\n\n    pdbout = str(a) + '_avgStruct.pdb'              # Name of the average PDB structure file\n\n    print(a, len(os.listdir(ARGS.splitdir)), pdbout, \"IF error happens\")\n\n    avg = AvgStructComputation(fastafile, kpax_dr, pdbout)\n    file = avg.compute_avg_struct()\n",
                            "writable": false
                        }
                    ],
                    "class": "InitialWorkDirRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "packages": [
                        {
                            "specs": [
                                "http://kpax.loria.fr/"
                            ],
                            "version": [
                                "5.1.3.x64"
                            ],
                            "package": "kpax"
                        }
                    ],
                    "class": "SoftwareRequirement"
                }
            ],
            "inputs": [
                {
                    "label": "Instance representing to all chopped domains",
                    "type": "File",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-f"
                    },
                    "id": "#align_compute_avg.cwl/fam_name"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "KPAX_RESULTS",
                    "inputBinding": {
                        "position": 4,
                        "prefix": "-k"
                    },
                    "id": "#align_compute_avg.cwl/kpax_result"
                },
                {
                    "type": [
                        "null",
                        "Directory",
                        "string"
                    ],
                    "label": "The direcory with PDB structures to compute average",
                    "default": {
                        "class": "Directory",
                        "location": "https://raw.githubusercontent.com/HrishiDhondge/CroMaSt/b5a9d4b025ec8e065bae97eeb96f10db2dd8e1e6/Tools/split_PDB",
                        "listing": []
                    },
                    "inputBinding": {
                        "position": 3,
                        "prefix": "-s"
                    },
                    "id": "#align_compute_avg.cwl/split_dir"
                }
            ],
            "baseCommand": [
                "python3",
                "align_compute_avg.py"
            ],
            "id": "#align_compute_avg.cwl",
            "https://schema.org/author": [
                {
                    "class": "https://schema.org/Person",
                    "https://schema.org/identifier": "https://orcid.org/0000-0002-7025-2241",
                    "https://schema.org/email": "mailto:hbdhondge@gmail.com",
                    "https://schema.org/name": "Hrishikesh Dhondge"
                },
                {
                    "class": "https://schema.org/Person",
                    "https://schema.org/identifier": "https://orcid.org/0000-0002-7035-3042",
                    "https://schema.org/email": "mailto:isaure.chauvot-de-beauchene@loria.fr",
                    "https://schema.org/name": "Isaure Chauvot de Beauch\u00eane"
                },
                {
                    "class": "https://schema.org/Person",
                    "https://schema.org/identifier": "https://orcid.org/0000-0002-0399-8713",
                    "https://schema.org/email": "mailto:marie-dominique.devignes@loria.fr",
                    "https://schema.org/name": "Marie-Dominique Devignes"
                }
            ],
            "https://schema.org/codeRepository": "https://gitlab.inria.fr/capsid.public_codes/CroMaSt",
            "https://schema.org/dateCreated": "2022-08-01",
            "https://schema.org/license": "https://mit-license.org/",
            "outputs": [
                {
                    "label": "Computed average structure for passed structures",
                    "type": "File",
                    "format": "http://edamontology.org/format_1476",
                    "outputBinding": {
                        "glob": "*.pdb"
                    },
                    "id": "#align_compute_avg.cwl/avg_structs"
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "label": "Compute and write average structure for given set of structures",
            "doc": "The tool chops the PDB files into domains according to given residue numebring and average structure from all chopped structures.\nThe input file is a json file where key is family id with and value is list of domain structures with residue numbering.\n",
            "requirements": [
                {
                    "envDef": [
                        {
                            "envValue": "$(inputs.kpax_result)",
                            "envName": "KPAX_RESULTS"
                        }
                    ],
                    "class": "EnvVarRequirement"
                },
                {
                    "listing": [
                        "$({class: \"Directory\", basename: inputs.pdb_dir.location, listing: []})",
                        "${ \n  if (typeof inputs.split_dir === 'string') {\n    return [\n      {\"class\": \"Directory\", basename: inputs.split_dir, listing: []}]; } \n  else { return [ inputs.split_dir] ; } \n }\n",
                        {
                            "entryname": "chop_struct2domains.py",
                            "entry": "#!/usr/bin/env python3\n\"\"\"\nThis script chops the PDB files into given domains according to the mapping provided in a different\ndirectory and aligns all the chooped structures using Kpax. Then the aligned structures are used to\ncompute the average structure.\nThis script requires a json file where key is family id with and value is list of domain structures\nwith residue numbering.\n\"\"\"\n\nimport argparse\nimport os\nimport json\nimport warnings\nfrom sh import gunzip\nimport wget\nimport Bio.PDB as bpdb\n\nwarnings.filterwarnings(\"ignore\")\n\n\nclass PDButil:\n    \"\"\"Class to provide some utilities for PDB files\"\"\"\n    def __init__(self, pdb_code, dir_pdb):\n        self.pdb_code = pdb_code.lower()\n        self.dir_pdb = dir_pdb\n\n    def get_pdb_file(self):\n        \"\"\"\n        Download and unzips the PDB file from rcsb in given directory\n        \"\"\"\n        url = 'https://files.rcsb.org/download/' + self.pdb_code + '.pdb.gz'\n        wget.download(url, out=self.dir_pdb)\n        if not os.path.join(self.dir_pdb, self.pdb_code + '.pdb') in os.listdir(self.dir_pdb):\n            gunzip(os.path.join(self.dir_pdb, self.pdb_code + '.pdb.gz'))\n\n    def get_cif_file(self):\n        \"\"\"\n        Download and unzips the PDB file from rcsb in given directory\n        \"\"\"\n        url = 'https://files.rcsb.org/download/' + self.pdb_code + '.cif'\n        if self.pdb_code.lower() + '.cif' not in os.listdir(self.dir_pdb):\n            wget.download(url, out=self.dir_pdb)\n\n    def pdb_chopper(self, chain_id, start_res, end_res, chopped, stype):\n        \"\"\"\n        Chops the PDB structure/file\n        \"\"\"\n        class ResSelect(bpdb.Select):\n            \"\"\"Re-Using the class from Bio.PDB\"\"\"\n            def accept_residue(self, res):\n                \"\"\"Actual chopping of the structure\"\"\"\n                return bool((res.id[1] >= int(start_res)) and (res.id[1] <= int(end_res)) and \\\n                    (res.parent.id == chain_id))\n\n        if stype == 'pdb':\n            wholepdb = os.path.join(self.dir_pdb, self.pdb_code + '.pdb')\n            stparse = bpdb.PDBParser().get_structure('temp', wholepdb)\n        elif stype in ('cif', 'mmcif'):\n            wholepdb = os.path.join(self.dir_pdb, self.pdb_code + '.cif')\n            stparse = bpdb.MMCIFParser().get_structure('temp', wholepdb)\n\n        inop = bpdb.PDBIO()\n\n        if len(chain_id) > 1:\n            stparse, chain_id = self.multi_letter_chains(stparse, chain_id)\n            inop.set_structure(stparse[chain_id])\n        else:\n            inop.set_structure(stparse[0][chain_id])\n        inop.save(chopped, ResSelect())\n        return chopped\n\n    @staticmethod\n    def multi_letter_chains(structure, chain_id):\n        \"\"\"\n        Assigns a single letter to the target chain, as only first letter of original id.\n        \"\"\"\n        model_first = structure[0] # first model\n        try: # Remove chain with first letter or target chain\n            model_first.detach_child(chain_id[0])\n        except:\n            pass\n        model_first[chain_id].id = chain_id[0]\n        return model_first, chain_id[0]\n\n\nclass UtilityDir:\n    \"\"\"Class to provide some utilities for PDB files\"\"\"\n    def __init__(self, adir):\n        self.dir = adir\n\n    def recursively_remove_files(self, dirpath=None):\n        \"\"\"USE WITH CAUTION:\n        Removes the directory even if it's not empty\"\"\"\n        dirpath = self.dir if dirpath is None else dirpath\n        if os.path.isfile(dirpath):\n            os.remove(dirpath)\n        elif os.path.isdir(dirpath):\n            for sub in os.listdir(dirpath):\n                self.recursively_remove_files(dirpath=os.path.join(self.dir, sub))\n            os.rmdir(dirpath)\n\n    def empty_dir(self):\n        \"\"\"Remove all files from a directory\"\"\"\n        for afile in os.listdir(self.dir):\n            try:\n                os.remove(os.path.join(self.dir, afile))\n            except:\n                print('Unable to delete ', afile)\n\n\n###################################################################################################\nif __name__ == '__main__':\n    P = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter,\\\n        epilog='Enjoy the program! :)')\n    P.add_argument(\"-f\", \"--file\", type=str, required=True, \\\n        help=\"File with list of StIs to chop into domains\")\n    P.add_argument(\"-p\", \"--pdbdir\", type=str, required=True, \\\n        help=\"Directory where all the PDB files are/will be stored\")\n    P.add_argument(\"-s\", \"--splitdir\", type=str, default=\"split_PDB/\", \\\n        help=\"Directory where all the chopped structures will be stored\")\n    P.add_argument(\"-k\", \"--kpax_res\", type=str, default=\"KPAX_RESULTS/\", \\\n        help=\"Directory path to store results from  kpax alignments\")\n\n    ARGS = P.parse_args()\n\n    file = ARGS.file\n    util_d = UtilityDir(ARGS.splitdir)\n    if os.path.exists(ARGS.splitdir):\n        util_d.recursively_remove_files()\n\n    for a in [ARGS.pdbdir, ARGS.splitdir]:\n        if not os.path.exists(a):\n            os.makedirs(a)\n\n    no_pdb, avg_structures = [], []\n    data = json.load(open(file, 'r'))\n\n    for a in data:\n        util_d.empty_dir() # removing files from previous iterations #split_dir\n\n        # For all mapped structures\n        for b in data[a]:\n            # reading each structure from given json file\n            struct = b.replace('\"', '').split(',')\n            pdb_id = struct[0].lower()\n            pdb = PDButil(pdb_id, ARGS.pdbdir)\n\n            out_file = os.path.join(ARGS.splitdir, '_'.join(struct[6:8]) + '_' + \\\n                '_'.join(struct[:2]) + '_' + '_'.join(struct[4:6]) + '.pdb')\n            if pdb_id + '.pdb' not in os.listdir(ARGS.pdbdir):\n                try:\n                    pdb.get_pdb_file()\n                except:\n                    try:\n                        pdb.get_cif_file()\n                        filename = pdb.pdb_chopper(struct[1], struct[4], struct[5], out_file, 'cif')\n                    except:\n                        no_pdb.append(pdb_id)\n                    continue\n\n            filename = pdb.pdb_chopper(struct[1], struct[4], struct[5], out_file, 'pdb')\n        print(a)\n",
                            "writable": false
                        }
                    ],
                    "class": "InitialWorkDirRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "packages": [
                        {
                            "specs": [
                                "http://kpax.loria.fr/"
                            ],
                            "version": [
                                "5.1.3.x64"
                            ],
                            "package": "kpax"
                        }
                    ],
                    "class": "SoftwareRequirement"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "KPAX_RESULTS",
                    "inputBinding": {
                        "position": 4,
                        "prefix": "-k"
                    },
                    "id": "#chop_struct2domains.cwl/kpax_result"
                },
                {
                    "type": [
                        "null",
                        "Directory"
                    ],
                    "label": "The directory containing all the PDB files",
                    "default": {
                        "class": "Directory",
                        "location": "https://raw.githubusercontent.com/HrishiDhondge/CroMaSt/b5a9d4b025ec8e065bae97eeb96f10db2dd8e1e6/PDB_files/",
                        "basename": "PDB_files"
                    },
                    "inputBinding": {
                        "position": 2,
                        "prefix": "-p"
                    },
                    "id": "#chop_struct2domains.cwl/pdb_dir"
                },
                {
                    "type": [
                        "Directory",
                        "string",
                        "null"
                    ],
                    "label": "The direcory name for storing splitted PDB structures",
                    "default": "split_PDB",
                    "inputBinding": {
                        "position": 3,
                        "prefix": "-s"
                    },
                    "id": "#chop_struct2domains.cwl/split_dir"
                },
                {
                    "label": "List of StIs to chop into domains",
                    "type": "File",
                    "format": "http://edamontology.org/format_3464",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-f"
                    },
                    "id": "#chop_struct2domains.cwl/struct_insta"
                }
            ],
            "outputs": [
                {
                    "label": "Instance representing to all chopped domains",
                    "type": "File",
                    "id": "#chop_struct2domains.cwl/family_name",
                    "outputBinding": {
                        "glob": "35389847c9da8e37379a6c703d8ca9bd823b088e"
                    }
                },
                {
                    "label": "Directory with all chopped domains",
                    "type": "Directory",
                    "outputBinding": {
                        "glob": "$(inputs.split_dir)"
                    },
                    "id": "#chop_struct2domains.cwl/split_structs_dir"
                }
            ],
            "baseCommand": [
                "python3",
                "chop_struct2domains.py"
            ],
            "id": "#chop_struct2domains.cwl",
            "https://schema.org/author": [
                {
                    "class": "https://schema.org/Person",
                    "https://schema.org/identifier": "https://orcid.org/0000-0002-7025-2241",
                    "https://schema.org/email": "mailto:hbdhondge@gmail.com",
                    "https://schema.org/name": "Hrishikesh Dhondge"
                },
                {
                    "class": "https://schema.org/Person",
                    "https://schema.org/identifier": "https://orcid.org/0000-0002-7035-3042",
                    "https://schema.org/email": "mailto:isaure.chauvot-de-beauchene@loria.fr",
                    "https://schema.org/name": "Isaure Chauvot de Beauch\u00eane"
                },
                {
                    "class": "https://schema.org/Person",
                    "https://schema.org/identifier": "https://orcid.org/0000-0002-0399-8713",
                    "https://schema.org/email": "mailto:marie-dominique.devignes@loria.fr",
                    "https://schema.org/name": "Marie-Dominique Devignes"
                }
            ],
            "https://schema.org/codeRepository": "https://gitlab.inria.fr/capsid.public_codes/CroMaSt",
            "https://schema.org/dateCreated": "2022-08-01",
            "https://schema.org/license": "https://mit-license.org/",
            "stdout": "35389847c9da8e37379a6c703d8ca9bd823b088e"
        },
        {
            "class": "Workflow",
            "label": "Compute average of average for core domain instances",
            "doc": "Compute average structure for all averaged structures corresponding to core UniProt domain instances.\nFirst computes average per UniProt domain instance and then average all averaged structures.\n",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "MultipleInputFeatureRequirement"
                },
                {
                    "class": "ScatterFeatureRequirement"
                },
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "SubworkflowFeatureRequirement"
                }
            ],
            "inputs": [
                {
                    "label": "Core domain StIs",
                    "type": "File",
                    "format": "http://edamontology.org/format_3464",
                    "id": "#main/core_list"
                },
                {
                    "label": "The directory with (or to store) PDB files",
                    "type": "Directory",
                    "id": "#main/pdb_dir"
                }
            ],
            "steps": [
                {
                    "label": "Compute average structure for all provided averaged structures",
                    "doc": "Align all the provided averaged structures using Kpax. And compute an average structure from all the aligned averaged structures,\nbased on aligned residues for at least 50% of the structures. \n",
                    "run": "#align_compute_avg.cwl",
                    "in": [
                        {
                            "source": "#main/per_unp_dom_instance/family_name",
                            "id": "#main/avg_averaged_structures/fam_name"
                        },
                        {
                            "source": "#main/copy_avg_dom/dir_unp_dom",
                            "id": "#main/avg_averaged_structures/split_dir"
                        }
                    ],
                    "out": [
                        "#main/avg_averaged_structures/avg_structs"
                    ],
                    "id": "#main/avg_averaged_structures"
                },
                {
                    "run": {
                        "class": "Workflow",
                        "inputs": [
                            {
                                "type": "File",
                                "id": "#main/avg_unp_domains/run/domfiles"
                            },
                            {
                                "type": "Directory",
                                "id": "#main/avg_unp_domains/run/pdb_storage"
                            }
                        ],
                        "steps": [
                            {
                                "label": "Compute average structure for each UniProt domain",
                                "doc": "Align all the provided structures using Kpax. And compute an average structure from all the aligned structural instances,\nbased on aligned residues for at least 50% of the instances. \n",
                                "run": "#align_compute_avg.cwl",
                                "in": [
                                    {
                                        "source": "#main/avg_unp_domains/run/chop_structs/family_name",
                                        "id": "#main/avg_unp_domains/run/avg_chopped_structs_unp_domains/fam_name"
                                    },
                                    {
                                        "source": "#main/avg_unp_domains/run/chop_structs/split_structs_dir",
                                        "id": "#main/avg_unp_domains/run/avg_chopped_structs_unp_domains/split_dir"
                                    }
                                ],
                                "out": [
                                    "#main/avg_unp_domains/run/avg_chopped_structs_unp_domains/avg_structs"
                                ],
                                "id": "#main/avg_unp_domains/run/avg_chopped_structs_unp_domains"
                            },
                            {
                                "run": "#chop_struct2domains.cwl",
                                "in": [
                                    {
                                        "source": "#main/avg_unp_domains/run/pdb_storage",
                                        "id": "#main/avg_unp_domains/run/chop_structs/pdb_dir"
                                    },
                                    {
                                        "source": "#main/avg_unp_domains/run/domfiles",
                                        "id": "#main/avg_unp_domains/run/chop_structs/struct_insta"
                                    }
                                ],
                                "out": [
                                    "#main/avg_unp_domains/run/chop_structs/split_structs_dir",
                                    "#main/avg_unp_domains/run/chop_structs/family_name"
                                ],
                                "id": "#main/avg_unp_domains/run/chop_structs"
                            }
                        ],
                        "outputs": [
                            {
                                "label": "Computed average structure for passed structures",
                                "type": "File",
                                "format": "http://edamontology.org/format_1476",
                                "outputSource": "#main/avg_unp_domains/run/avg_chopped_structs_unp_domains/avg_structs",
                                "id": "#main/avg_unp_domains/run/avg_unp_dom_structs"
                            }
                        ]
                    },
                    "scatter": "#main/avg_unp_domains/domfiles",
                    "in": [
                        {
                            "source": "#main/per_unp_dom_instance/dom_per_fam",
                            "id": "#main/avg_unp_domains/domfiles"
                        },
                        {
                            "source": "#main/pdb_dir",
                            "id": "#main/avg_unp_domains/pdb_storage"
                        }
                    ],
                    "out": [
                        "#main/avg_unp_domains/avg_unp_dom_structs"
                    ],
                    "id": "#main/avg_unp_domains"
                },
                {
                    "run": "#move_files.cwl",
                    "in": [
                        {
                            "source": "#main/avg_unp_domains/avg_unp_dom_structs",
                            "id": "#main/copy_avg_dom/avg_unp_dom"
                        }
                    ],
                    "out": [
                        "#main/copy_avg_dom/dir_unp_dom"
                    ],
                    "id": "#main/copy_avg_dom"
                },
                {
                    "run": "#crossmapped_per_unp_dom.cwl",
                    "in": [
                        {
                            "source": "#main/core_list",
                            "id": "#main/per_unp_dom_instance/fam_structs"
                        }
                    ],
                    "out": [
                        "#main/per_unp_dom_instance/dom_per_fam",
                        "#main/per_unp_dom_instance/family_name"
                    ],
                    "id": "#main/per_unp_dom_instance"
                }
            ],
            "outputs": [
                {
                    "label": "Core average structure",
                    "type": "File",
                    "format": "http://edamontology.org/format_1476",
                    "outputSource": "#main/avg_averaged_structures/avg_structs",
                    "id": "#main/averaged_structs"
                }
            ],
            "id": "#main",
            "https://schema.org/author": [
                {
                    "class": "https://schema.org/Person",
                    "https://schema.org/identifier": "https://orcid.org/0000-0002-7025-2241",
                    "https://schema.org/email": "mailto:hbdhondge@gmail.com",
                    "https://schema.org/name": "Hrishikesh Dhondge"
                },
                {
                    "class": "https://schema.org/Person",
                    "https://schema.org/identifier": "https://orcid.org/0000-0002-7035-3042",
                    "https://schema.org/email": "mailto:isaure.chauvot-de-beauchene@loria.fr",
                    "https://schema.org/name": "Isaure Chauvot de Beauch\u00eane"
                },
                {
                    "class": "https://schema.org/Person",
                    "https://schema.org/identifier": "https://orcid.org/0000-0002-0399-8713",
                    "https://schema.org/email": "mailto:marie-dominique.devignes@loria.fr",
                    "https://schema.org/name": "Marie-Dominique Devignes"
                }
            ],
            "https://schema.org/codeRepository": "https://gitlab.inria.fr/capsid.public_codes/CroMaSt",
            "https://schema.org/dateCreated": "2022-08-01",
            "https://schema.org/license": "https://mit-license.org/"
        },
        {
            "class": "CommandLineTool",
            "label": "Divides a dictionary of structures per family into multiple according to domain instance",
            "doc": "The tool takes a dictionary and divides all the structures according to their UniProt id and domain position.\n",
            "requirements": [
                {
                    "listing": [
                        {
                            "entryname": "crossmapped_per_unp_dom.py",
                            "entry": "#!/usr/bin/env python3\nimport json, sys, os\nall_data = json.load(open('$(inputs.fam_structs.path)', 'r'))\ndone_dom = {}\n# This is designed only for first key in the dict/json \nif len(all_data.keys()) < 1:\n    sys.exit()\nfam_name = list(all_data.keys())[0]\nfor struct in all_data[fam_name]:\n    struct = struct.split(',')\n    if fam_name + '_' + '_'.join(struct[6:8]) in done_dom:\n        done_dom[fam_name + '_' + '_'.join(struct[6:8])].append(','.join(struct))\n    else:\n        done_dom[fam_name + '_' + '_'.join(struct[6:8])] = [','.join(struct)]\n\nfor domx in done_dom:\n    tmp_dom = {domx: done_dom[domx]}\n    with open(domx + '.json', 'w') as f:\n        json.dump(tmp_dom, f, indent=2)\nprint(fam_name)",
                            "writable": false
                        }
                    ],
                    "class": "InitialWorkDirRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "inputs": [
                {
                    "label": "Cross-mapped domain StIs per family",
                    "type": "File",
                    "format": "http://edamontology.org/format_3464",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-f"
                    },
                    "id": "#crossmapped_per_unp_dom.cwl/fam_structs"
                }
            ],
            "outputs": [
                {
                    "label": "List of StIs per UniProt domain instance",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "format": "http://edamontology.org/format_3464",
                    "outputBinding": {
                        "glob": "*.json"
                    },
                    "id": "#crossmapped_per_unp_dom.cwl/dom_per_fam"
                },
                {
                    "label": "UniProt domain instance corresponding to StIs",
                    "type": "File",
                    "id": "#crossmapped_per_unp_dom.cwl/family_name",
                    "outputBinding": {
                        "glob": "7a66558b2f73b2708851265114bc777c25b4c568"
                    }
                }
            ],
            "baseCommand": [
                "python3",
                "crossmapped_per_unp_dom.py"
            ],
            "id": "#crossmapped_per_unp_dom.cwl",
            "https://schema.org/author": [
                {
                    "class": "https://schema.org/Person",
                    "https://schema.org/identifier": "https://orcid.org/0000-0002-7025-2241",
                    "https://schema.org/email": "mailto:hbdhondge@gmail.com",
                    "https://schema.org/name": "Hrishikesh Dhondge"
                },
                {
                    "class": "https://schema.org/Person",
                    "https://schema.org/identifier": "https://orcid.org/0000-0002-7035-3042",
                    "https://schema.org/email": "mailto:isaure.chauvot-de-beauchene@loria.fr",
                    "https://schema.org/name": "Isaure Chauvot de Beauch\u00eane"
                },
                {
                    "class": "https://schema.org/Person",
                    "https://schema.org/identifier": "https://orcid.org/0000-0002-0399-8713",
                    "https://schema.org/email": "mailto:marie-dominique.devignes@loria.fr",
                    "https://schema.org/name": "Marie-Dominique Devignes"
                }
            ],
            "https://schema.org/codeRepository": "https://gitlab.inria.fr/capsid.public_codes/CroMaSt",
            "https://schema.org/dateCreated": "2022-08-01",
            "https://schema.org/license": "https://mit-license.org/",
            "stdout": "7a66558b2f73b2708851265114bc777c25b4c568"
        },
        {
            "class": "CommandLineTool",
            "label": "Moves passed files to given directory",
            "doc": "The tool copy all the files from their original location to the directory provided by user.\n",
            "requirements": [
                {
                    "listing": [
                        "${ \n  if (typeof inputs.dir_cp === 'string') {\n    return [\n      {\"class\": \"Directory\", basename: inputs.dir_cp, listing: []}]; } \n  else { return [ inputs.dir_cp] ; } \n }\n",
                        {
                            "entryname": "script.py",
                            "entry": "#!/usr/bin/env python3\nimport json, os\nfor i in $(inputs.avg_unp_dom):\n    if not isinstance(i, dict):\n        continue\n    cmd = 'cp {0} {1}'.format(i['path'], '$(inputs.dir_cp)')\n    os.system(cmd)",
                            "writable": false
                        }
                    ],
                    "class": "InitialWorkDirRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "inputs": [
                {
                    "label": "Files to move into new direcory",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "format": "http://edamontology.org/format_1476",
                    "id": "#move_files.cwl/avg_unp_dom"
                },
                {
                    "type": [
                        "Directory",
                        "string",
                        "null"
                    ],
                    "label": "The direcory name for storing avg_split PDB structures",
                    "default": "avg_split_PDB",
                    "id": "#move_files.cwl/dir_cp"
                }
            ],
            "outputs": [
                {
                    "type": "Directory",
                    "outputBinding": {
                        "glob": "$(inputs.dir_cp)"
                    },
                    "id": "#move_files.cwl/dir_unp_dom"
                }
            ],
            "baseCommand": [
                "python",
                "script.py"
            ],
            "id": "#move_files.cwl",
            "https://schema.org/author": [
                {
                    "class": "https://schema.org/Person",
                    "https://schema.org/identifier": "https://orcid.org/0000-0002-7025-2241",
                    "https://schema.org/email": "mailto:hbdhondge@gmail.com",
                    "https://schema.org/name": "Hrishikesh Dhondge"
                },
                {
                    "class": "https://schema.org/Person",
                    "https://schema.org/identifier": "https://orcid.org/0000-0002-7035-3042",
                    "https://schema.org/email": "mailto:isaure.chauvot-de-beauchene@loria.fr",
                    "https://schema.org/name": "Isaure Chauvot de Beauch\u00eane"
                },
                {
                    "class": "https://schema.org/Person",
                    "https://schema.org/identifier": "https://orcid.org/0000-0002-0399-8713",
                    "https://schema.org/email": "mailto:marie-dominique.devignes@loria.fr",
                    "https://schema.org/name": "Marie-Dominique Devignes"
                }
            ],
            "https://schema.org/codeRepository": "https://gitlab.inria.fr/capsid.public_codes/CroMaSt",
            "https://schema.org/dateCreated": "2022-08-01",
            "https://schema.org/license": "https://mit-license.org/"
        }
    ],
    "cwlVersion": "v1.2",
    "$schemas": [
        "https://schema.org/version/latest/schemaorg-current-https.rdf",
        "http://edamontology.org/EDAM_1.18.owl"
    ],
    "$namespaces": {
        "s": "https://schema.org/",
        "edam": "http://edamontology.org/"
    }
}
